@@ -10,9 +10,21 @@
 # propagated, or distributed except according to the terms contained in the
 # LICENSE file.
 
-"""Bitcoin Core RPC support"""
+"""Bitcoin Core RPC support
+
+By default this uses the standard library ``json`` module. By monkey patching,
+a different implementation can be used instead, at your own risk:
+
+>>> import simplejson
+>>> import bitcoin.rpc
+>>> bitcoin.rpc.json = simplejson
+
+(``simplejson`` is the externally maintained version of the same module and
+thus better optimized but perhaps less stable.)
+"""
 
 from __future__ import absolute_import, division, print_function, unicode_literals
+import ssl
 
 try:
     import http.client as httplib
@@ -47,22 +59,29 @@ if sys.version > '3':
     hexlify = lambda b: binascii.hexlify(b).decode('utf8')
 
 
-class JSONRPCException(Exception):
+class JSONRPCError(Exception):
+    """JSON-RPC protocol error"""
+
     def __init__(self, rpc_error):
-        super(JSONRPCException, self).__init__('msg: %r  code: %r' %
-                (rpc_error['message'], rpc_error['code']))
+        super(JSONRPCException, self).__init__(
+            'msg: %r  code: %r' %
+            (rpc_error['message'], rpc_error['code']))
         self.error = rpc_error
 
 
-class RawProxy(object):
-    def __init__(self, service_url=None,
-                       service_port=None,
-                       btc_conf_file=None,
-                       timeout=DEFAULT_HTTP_TIMEOUT):
-        """Low-level JSON-RPC proxy
+# 0.4.0 compatibility
+JSONRPCException = JSONRPCError
 
-        Unlike Proxy no conversion is done from the raw JSON objects.
-        """
+
+class BaseProxy(object):
+    """Base JSON-RPC proxy class. Contains only private methods; do not use
+    directly."""
+
+    def __init__(self,
+                 service_url=None,
+                 service_port=None,
+                 btc_conf_file=None,
+                 timeout=DEFAULT_HTTP_TIMEOUT):
 
         if service_url is None:
             # Figure out the path to the bitcoin.conf file
@@ -91,6 +110,7 @@ class RawProxy(object):
                     service_port = bitcoin.params.RPC_PORT
                 conf['rpcport'] = int(conf.get('rpcport', service_port))
                 conf['rpcssl'] = conf.get('rpcssl', '0')
+                conf['rpchost'] = conf.get('rpcconnect', 'localhost')
 
                 if conf['rpcssl'].lower() in ('0', 'false'):
                     conf['rpcssl'] = False
@@ -99,13 +119,29 @@ class RawProxy(object):
                 else:
                     raise ValueError('Unknown rpcssl value %r' % conf['rpcssl'])
 
+                if conf['rpcssl'] and 'rpcsslcertificatechainfile' in conf and 'rpcsslprivatekeyfile' in conf:
+                    self.__ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
+                    if os.path.exists(conf['rpcsslcertificatechainfile']):
+                        certificate = conf['rpcsslcertificatechainfile']
+                    elif os.path.exists(os.path.join(os.path.dirname(btc_conf_file), conf['rpcsslcertificatechainfile'])):
+                        certificate = os.path.join(os.path.dirname(btc_conf_file), conf['rpcsslcertificatechainfile'])
+                    else:
+                        raise ValueError('The value of rpcsslcertificatechainfile is not correctly specified in the configuration file: %s' % btc_conf_file)
+                    if os.path.exists(conf['rpcsslprivatekeyfile']):
+                        private_key = conf['rpcsslprivatekeyfile']
+                    elif os.path.exists(os.path.join(os.path.dirname(btc_conf_file), conf['rpcsslprivatekeyfile'])):
+                        private_key = os.path.join(os.path.dirname(btc_conf_file), conf['rpcsslprivatekeyfile'])
+                    else:
+                        raise ValueError('The value of rpcsslprivatekeyfile is not correctly specified in the configuration file: %s' % btc_conf_file)
+                    self.__ssl_context.load_cert_chain(certificate, private_key)
+
                 if 'rpcpassword' not in conf:
                     raise ValueError('The value of rpcpassword not specified in the configuration file: %s' % btc_conf_file)
 
-                service_url = ('%s://%s:%s@localhost:%d' %
+                service_url = ('%s://%s:%s@%s:%d' %
                     ('https' if conf['rpcssl'] else 'http',
                      conf['rpcuser'], conf['rpcpassword'],
-                     conf['rpcport']))
+                     conf['rpchost'], conf['rpcport']))
 
         self.__service_url = service_url
         self.__url = urlparse.urlparse(service_url)
@@ -127,7 +163,7 @@ class RawProxy(object):
 
         if self.__url.scheme == 'https':
             self.__conn = httplib.HTTPSConnection(self.__url.hostname, port=port,
-                                                  key_file=None, cert_file=None,
+                                                  context=self.__ssl_context,
                                                   timeout=timeout)
         else:
             self.__conn = httplib.HTTPConnection(self.__url.hostname, port=port,
@@ -149,28 +185,14 @@ class RawProxy(object):
 
         response = self._get_response()
         if response['error'] is not None:
-            raise JSONRPCException(response['error'])
+            raise JSONRPCError(response['error'])
         elif 'result' not in response:
-            raise JSONRPCException({
+            raise JSONRPCError({
                 'code': -343, 'message': 'missing JSON-RPC result'})
         else:
             return response['result']
 
 
-    def __getattr__(self, name):
-        if name.startswith('__') and name.endswith('__'):
-            # Python internal stuff
-            raise AttributeError
-
-        # Create a callable to do the actual call
-        f = lambda *args: self._call(name, *args)
-
-        # Make debuggers show <function bitcoin.rpc.name> rather than <function
-        # bitcoin.rpc.<lambda>>
-        f.__name__ = name
-        return f
-
-
     def _batch(self, rpc_call_list):
         postdata = json.dumps(list(rpc_call_list))
         self.__conn.request('POST', self.__url.path, postdata,
@@ -184,7 +206,7 @@ class RawProxy(object):
     def _get_response(self):
         http_response = self.__conn.getresponse()
         if http_response is None:
-            raise JSONRPCException({
+            raise JSONRPCError({
                 'code': -342, 'message': 'missing HTTP response from server'})
 
         return json.loads(http_response.read().decode('utf8'),
@@ -194,33 +216,79 @@ class RawProxy(object):
         self.__conn.close()
 
 
-class Proxy(RawProxy):
-    def __init__(self, service_url=None,
-                       service_port=None,
-                       btc_conf_file=None,
-                       timeout=DEFAULT_HTTP_TIMEOUT,
-                       **kwargs):
-        """Create a proxy to a bitcoin RPC service
+class RawProxy(BaseProxy):
+    """Low-level proxy to a bitcoin JSON-RPC service
 
-        Unlike RawProxy data is passed as objects, rather than JSON. (not yet
-        fully implemented) Assumes Bitcoin Core version >= 0.9; older versions
-        mostly work, but there are a few incompatibilities.
+    Unlike ``Proxy``, no conversion is done besides parsing JSON. As far as
+    Python is concerned, you can call any method; ``JSONRPCError`` will be
+    raised if the server does not recognize it.
+    """
+    def __init__(self,
+                 service_url=None,
+                 service_port=None,
+                 btc_conf_file=None,
+                 timeout=DEFAULT_HTTP_TIMEOUT,
+                 **kwargs):
+        super(RawProxy, self).__init__(service_url=service_url,
+                                       service_port=service_port,
+                                       btc_conf_file=btc_conf_file,
+                                       timeout=timeout,
+                                       **kwargs)
 
-        If service_url is not specified the username and password are read out
-        of the file btc_conf_file. If btc_conf_file is not specified
-        ~/.bitcoin/bitcoin.conf or equivalent is used by default. The default
-        port is set according to the chain parameters in use: mainnet, testnet,
-        or regtest.
+    def __getattr__(self, name):
+        if name.startswith('__') and name.endswith('__'):
+            # Python internal stuff
+            raise AttributeError
 
-        Usually no arguments to Proxy() are needed; the local bitcoind will be
-        used.
+        # Create a callable to do the actual call
+        f = lambda *args: self._call(name, *args)
 
-        timeout - timeout in seconds before the HTTP interface times out
+        # Make debuggers show <function bitcoin.rpc.name> rather than <function
+        # bitcoin.rpc.<lambda>>
+        f.__name__ = name
+        return f
+
+
+class Proxy(BaseProxy):
+    """Proxy to a bitcoin RPC service
+
+    Unlike ``RawProxy``, data is passed as ``bitcoin.core`` objects or packed
+    bytes, rather than JSON or hex strings. Not all methods are implemented
+    yet; you can use ``call`` to access missing ones in a forward-compatible
+    way. Assumes Bitcoin Core version >= 0.9; older versions mostly work, but
+    there are a few incompatibilities.
+    """
+
+    def __init__(self,
+                 service_url=None,
+                 service_port=None,
+                 btc_conf_file=None,
+                 timeout=DEFAULT_HTTP_TIMEOUT,
+                 **kwargs):
+        """Create a proxy object
+
+        If ``service_url`` is not specified, the username and password are read
+        out of the file ``btc_conf_file``. If ``btc_conf_file`` is not
+        specified, ``~/.bitcoin/bitcoin.conf`` or equivalent is used by
+        default.  The default port is set according to the chain parameters in
+        use: mainnet, testnet, or regtest.
+
+        Usually no arguments to ``Proxy()`` are needed; the local bitcoind will
+        be used.
+
+        ``timeout`` - timeout in seconds before the HTTP interface times out
         """
-        super(Proxy, self).__init__(service_url=service_url, service_port=service_port, btc_conf_file=btc_conf_file,
+
+        super(Proxy, self).__init__(service_url=service_url,
+                                    service_port=service_port,
+                                    btc_conf_file=btc_conf_file,
                                     timeout=timeout,
                                     **kwargs)
 
+    def call(self, service_name, *args):
+        """Call an RPC method by name and raw (JSON encodable) arguments"""
+        return self._call(service_name, *args)
+
     def dumpprivkey(self, addr):
         """Return the private key matching an address
         """
@@ -229,19 +297,27 @@ class Proxy(RawProxy):
         return CBitcoinSecret(r)
 
     def getaccountaddress(self, account=None):
-        """Return the current Bitcoin address for receiving payments to this account."""
+        """Return the current Bitcoin address for receiving payments to this
+        account."""
         r = self._call('getaccountaddress', account)
         return CBitcoinAddress(r)
 
     def getbalance(self, account='*', minconf=1):
         """Get the balance
 
-        account - The selected account. Defaults to "*" for entire wallet. It may be the default account using "".
-        minconf - Only include transactions confirmed at least this many times. (default=1)
+        account - The selected account. Defaults to "*" for entire wallet. It
+        may be the default account using "".
+
+        minconf - Only include transactions confirmed at least this many times.
+        (default=1)
         """
         r = self._call('getbalance', account, minconf)
         return int(r*COIN)
 
+    def getbestblockhash(self):
+        """Return hash of best (tip) block in longest block chain."""
+        return lx(self._call('getbestblockhash'))
+
     def getblock(self, block_hash):
         """Get block <block_hash>
 
@@ -254,11 +330,15 @@ class Proxy(RawProxy):
                     (self.__class__.__name__, block_hash.__class__))
         try:
             r = self._call('getblock', block_hash, False)
-        except JSONRPCException as ex:
+        except JSONRPCError as ex:
             raise IndexError('%s.getblock(): %s (%d)' %
                     (self.__class__.__name__, ex.error['message'], ex.error['code']))
         return CBlock.deserialize(unhexlify(r))
 
+    def getblockcount(self):
+        """Return the number of blocks in the longest block chain"""
+        return self._call('getblockcount')
+
     def getblockhash(self, height):
         """Return hash of block in best-block-chain at height.
 
@@ -266,18 +346,23 @@ class Proxy(RawProxy):
         """
         try:
             return lx(self._call('getblockhash', height))
-        except JSONRPCException as ex:
+        except JSONRPCError as ex:
             raise IndexError('%s.getblockhash(): %s (%d)' %
                     (self.__class__.__name__, ex.error['message'], ex.error['code']))
 
     def getinfo(self):
-        """Return an object containing various state info"""
+        """Return a JSON object containing various state info"""
         r = self._call('getinfo')
         if 'balance' in r:
             r['balance'] = int(r['balance'] * COIN)
-        r['paytxfee'] = int(r['paytxfee'] * COIN)
+        if 'paytxfee' in r:
+            r['paytxfee'] = int(r['paytxfee'] * COIN)
         return r
 
+    def getmininginfo(self):
+        """Return a JSON object containing mining-related information"""
+        return self._call('getmininginfo')
+
     def getnewaddress(self, account=None):
         """Return a new Bitcoin address for receiving payments.
 
@@ -315,15 +400,15 @@ class Proxy(RawProxy):
 
         Raises IndexError if transaction not found.
 
-        verbse - If true a dict is returned instead with additional information
-                 on the transaction.
+        verbose - If true a dict is returned instead with additional
+        information on the transaction.
 
         Note that if all txouts are spent and the transaction index is not
         enabled the transaction may not be available.
         """
         try:
             r = self._call('getrawtransaction', b2lx(txid), 1 if verbose else 0)
-        except JSONRPCException as ex:
+        except JSONRPCError as ex:
             raise IndexError('%s.getrawtransaction(): %s (%d)' %
                     (self.__class__.__name__, ex.error['message'], ex.error['code']))
         if verbose:
@@ -350,7 +435,9 @@ class Proxy(RawProxy):
         always show zero.
 
         addr    - The address. (CBitcoinAddress instance)
-        minconf - Only include transactions confirmed at least this many times. (default=1)
+
+        minconf - Only include transactions confirmed at least this many times.
+        (default=1)
         """
         r = self._call('getreceivedbyaddress', str(addr), minconf)
         return int(r * COIN)
@@ -364,7 +451,7 @@ class Proxy(RawProxy):
         """
         try:
             r = self._call('gettransaction', b2lx(txid))
-        except JSONRPCException as ex:
+        except JSONRPCError as ex:
             raise IndexError('%s.getrawtransaction(): %s (%d)' %
                     (self.__class__.__name__, ex.error['message'], ex.error['code']))
         return r
@@ -388,17 +475,11 @@ class Proxy(RawProxy):
         r['bestblock'] = lx(r['bestblock'])
         return r
 
-    def importaddress(self, addr, label=None, rescan=True):
+    def importaddress(self, addr, label='', rescan=True):
         """Adds an address or pubkey to wallet without the associated privkey."""
         addr = str(addr)
 
-        if label is not None:
-            if rescan:
-                r = self._call('importaddress', addr, label, True)
-            else:
-                r = self._call('importaddress', addr, label)
-        else:
-            r = self._call('importaddress', addr)
+        r = self._call('importaddress', addr, label, rescan)
         return r
 
     def listunspent(self, minconf=0, maxconf=9999999, addrs=None):
@@ -429,7 +510,8 @@ class Proxy(RawProxy):
 
     def lockunspent(self, unlock, outpoints):
         """Lock or unlock outpoints"""
-        json_outpoints = [{'txid':b2lx(outpoint.hash),'vout':outpoint.n} for outpoint in outpoints]
+        json_outpoints = [{'txid':b2lx(outpoint.hash), 'vout':outpoint.n}
+                          for outpoint in outpoints]
         return self._call('lockunspent', unlock, json_outpoints)
 
     def sendrawtransaction(self, tx, allowhighfees=False):
@@ -447,7 +529,8 @@ class Proxy(RawProxy):
 
     def sendmany(self, fromaccount, payments, minconf=1, comment=''):
         """Sent amount to a given address"""
-        json_payments = {str(addr):float(amount)/COIN for addr,amount in payments.items()}
+        json_payments = {str(addr):float(amount)/COIN
+                         for addr, amount in payments.items()}
         r = self._call('sendmany', fromaccount, json_payments, minconf, comment)
         return lx(r)
 
@@ -504,7 +587,8 @@ class Proxy(RawProxy):
         return self._addnode(node, 'remove')
 
 __all__ = (
-        'JSONRPCException',
-        'RawProxy',
-        'Proxy',
+    'JSONRPCError',
+    'JSONRPCException',
+    'RawProxy',
+    'Proxy',
 )
