@@ -107,3 +107,30 @@ class Test_BytesSerializer(unittest.TestCase):
         T(b'01')
         T(b'0200')
         T(b'ff00000000000000ff11223344', SerializationError) # > max_size
+
+class Test_Compact(unittest.TestCase):
+    def test_from_compact_zero(self):
+        self.assertEqual(uint256_from_compact(0x00123456), 0)
+        self.assertEqual(uint256_from_compact(0x01003456), 0)
+        self.assertEqual(uint256_from_compact(0x02000056), 0)
+        self.assertEqual(uint256_from_compact(0x03000000), 0)
+        self.assertEqual(uint256_from_compact(0x04000000), 0)
+        self.assertEqual(uint256_from_compact(0x00923456), 0)
+    def test_from_compact_negative_zero(self):
+        # Negative bit isn't supported yet
+        # self.assertEqual(uint256_from_compact(0x01803456), 0)
+        # self.assertEqual(uint256_from_compact(0x02800056), 0)
+        # self.assertEqual(uint256_from_compact(0x03800000), 0)
+        # self.assertEqual(uint256_from_compact(0x04800000), 0)
+        return
+
+    def test_twelve(self):
+        self.assertEqual(uint256_from_compact(0x01123456), 0x0012)
+        self.assertEqual(compact_from_uint256(0x0012), 0x01120000)
+
+    def test_from_uint256(self):
+        self.assertEqual(compact_from_uint256(0x1234), 0x02123400)
+        self.assertEqual(compact_from_uint256(0x123456), 0x03123456)
+        self.assertEqual(compact_from_uint256(0x12345600), 0x04123456)
+        self.assertEqual(compact_from_uint256(0x92340000), 0x05009234)
+        self.assertEqual(compact_from_uint256(0x1234560000000000000000000000000000000000000000000000000000000000), 0x20123456)
